// SPDX-License-Identifier: GPL-2.0-only
/*
 *  add audit function based on extend attribute security.seaudit
 *
 *  This file contains the selinux hooks2 function implementations.
 *
 *  Authors:  zhoupeng, <zhoupeng@uniontech.com>
 *
 *  Copyright (C) 2022 UnionTech Technology, Inc.
 */
#include <linux/socket.h>
#ifndef __LINUX_CONFIG_SECURITY2_H
#define __LINUX_CONFIG_SECURITY2_H

typedef enum {
	SECURITY_E_FILE_CREATE =1,
	SECURITY_E_DIR_CREATE,
	SECURITY_E_SYMLINK_CREATE
}SECURITY_E_TYPE;

#ifdef CONFIG_SELINUX_SID2
//below extended for selinux for audit
extern int g_seaudit_started;
#define security_file_close(file) \
	do {\
	   if (g_seaudit_started) \
		security_file_close_real(file);\
	}\
	while(0)

#define security_file_write(file, count) \
	do {\
	if(g_seaudit_started) \
		security_file_write_real(file,count);\
	}\
	while(0)

#define security_file_read(file, count) \
	do {\
		if (g_seaudit_started)\
			security_file_read_real(file, count);\
	}\
	while(0)

#define security_process_exit(task) \
	do {\
		if (g_seaudit_started) \
			security_process_exit_real(task);\
	}\
	while(0)

#define security_tcp_recvmsg(sk, msg, copied) \
	do {\
		if (g_seaudit_started) \
			security_tcp_recvmsg_real(sk, msg,copied);\
	}\
	while(0)

#define security_udp_sendmsg(sk, msg, saddr, sport, daddr, dport, sent) \
	do {\
		if (g_seaudit_started) \
			security_udp_sendmsg_real(sk, msg, saddr, sport, daddr, dport, sent);\
	}\
	while(0)

#define security_udp_recvmsg(sk, msg, saddr, sport, daddr, dport, recved) \
	do {\
		if(g_seaudit_started) \
			security_udp_recvmsg_real(sk, msg, saddr, sport, daddr, dport, recved);\
	}\
	while(0)

#define security_file_create(dir_inode, dentry, op) \
	do {\
		if (g_seaudit_started) \
			security_file_create_real(dir_inode, dentry, op);\
	}\
	while(0)

#define security_socket_accept_post(sock, newsock) \
		do {\
			if (g_seaudit_started) \
				security_socket_accept_post_real(sock,newsock);\
		}\
		while(0)

int security_file_close_real(struct file* file);
int security_process_exit_real(struct task_struct* task);
int security_tcp_recvmsg_real(struct sock* sk, struct msghdr *msg, size_t copied);
int security_udp_sendmsg_real(struct sock*sk, struct msghdr *msg, u32 saddr, u16 sport, u32 daddr, u16 dport, size_t sent);
int security_udp_recvmsg_real(struct sock*sk, struct msghdr *msg, u32 saddr, u16 sport, u32 daddr, u16 dport, size_t recved);
int security_file_create_real(const struct inode *dir_inode, struct dentry *dentry, SECURITY_E_TYPE op);

int security_socket_accept_post_real(struct socket* sock, struct socket *newsock);
void security_set_audit_started(int started);

int security_file_write_real(struct file* file, size_t count);
int security_file_read_real(struct file* file, size_t count);
int security_socket_getpeersec2_stream(struct socket *sock, char __user *optval,
				       int __user *optlen, unsigned len);
#else
static inline int security_file_close(struct file* file)
{
	return 0;
}

static inline int security_file_write(struct file* file, size_t count)
{
	return 0;
}

static inline int security_file_read(struct file* file, size_t count)
{
	return 0;
}

static inline int security_process_exit(struct task_struct* task)
{
	return 0;
}

static inline int security_tcp_recvmsg(struct sock* sk, struct msghdr *msg, size_t copied)
{
	return 0;
}
static inline int security_udp_sendmsg(struct sock*sk, struct msghdr *msg,
            u32 saddr, u16 sport, u32 daddr, u16 dport, size_t sent)
{
	return 0;
}

static inline int security_udp_recvmsg(struct sock*sk, struct msghdr *msg,
            u32 saddr, u16 sport, u32 daddr, u16 dport, size_t sent)
{
	return 0;
}

static inline int security_socket_accept_post(struct socket *sock, struct socket *newsock)
{
	return 0;
}

static inline int security_file_create(const struct inode *dir_inode, struct dentry *dentry, SECURITY_E_TYPE op)
{
	return 0;
}

static inline int security_socket_getpeersec2_stream(struct socket *sock, char __user *optval,
				       int __user *optlen, unsigned len)
{
	return -ENOPROTOOPT;
}

void security_set_audit_started(int started)
{
}
#endif
#endif
